"use strict";
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
var validation_result_1 = require("./validation-result");
__export(require("./validation-result"));
function validate(arg, assertion) {
    return utils_1.tryCatch(function () { return assertion(arg); }, function (err) { return validation_result_1.errorFromException(err); });
}
exports.validate = validate;
function extendValidator(validator1, validator2) {
    var result = {};
    for (var key in validator1) {
        result[key] = validator1[key];
    }
    for (var key in validator2) {
        result[key] = validator2[key];
    }
    return result;
}
exports.extendValidator = extendValidator;
function conformsTo(validator, optionsOrNext, next) {
    return isObject(function (arg) {
        var options = typeof optionsOrNext === 'object' ? optionsOrNext : {};
        var nextAssertion = typeof optionsOrNext === 'function' ? optionsOrNext : next;
        var _a = options.allowAdditionalProperties, allowAdditionalProperties = _a === void 0 ? true : _a;
        var partiallyValidated = {};
        var errors = utils_1.keysOf(validator).reduce(function (errors, key) {
            var result = utils_1.tryCatch(function () { return validator[key](arg[key]); }, function (err) { return validation_result_1.errorFromException(err); });
            if (!result.success) {
                return errors.concat(result.addPathNode(new validation_result_1.KeyPathNode(key)).errors);
            }
            partiallyValidated[key] = result.value;
            return errors;
        }, []);
        if (!allowAdditionalProperties) {
            var additionalProperties = utils_1.keysOf(arg).filter(function (key) { return !validator.hasOwnProperty(key); });
            if (additionalProperties.length > 0) {
                errors.push(new validation_result_1.ValidationError('UNEXPECTED_ADDITIONAL_PROPERTIES', "Unexpected additional propert" + (additionalProperties.length === 1 ? 'y' : 'ies') + ": " + additionalProperties.join(', ')));
            }
        }
        if (errors.length > 0)
            return new validation_result_1.ErrorResult(errors);
        var validated = partiallyValidated;
        return next ? next(validated) : validation_result_1.success(validated);
    });
}
exports.conformsTo = conformsTo;
function optional(next) {
    return function (arg) {
        if (arg === undefined)
            return validation_result_1.success(undefined);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.optional = optional;
function nullable(next) {
    return function (arg) {
        if (arg === null)
            return validation_result_1.success(null);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.nullable = nullable;
function defaultsTo(def, next) {
    return function (arg) {
        if (arg === undefined)
            arg = def;
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.defaultsTo = defaultsTo;
function onErrorDefaultsTo(def, next) {
    return function (arg) {
        var result = utils_1.tryCatch(function () { return next(arg); }, function (err) { return validation_result_1.errorFromException(err); });
        if (result.success)
            return result;
        // Ignore error - resort to default
        return validation_result_1.success(def);
    };
}
exports.onErrorDefaultsTo = onErrorDefaultsTo;
function isBoolean(next) {
    return function (arg) {
        if (typeof arg !== 'boolean')
            return validation_result_1.error('NOT_BOOLEAN', "Expected boolean, got " + typeof arg);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.isBoolean = isBoolean;
function isNumber(next) {
    return function (arg) {
        if (typeof arg !== 'number')
            return validation_result_1.error('NOT_NUMBER', "Expected number, got " + typeof arg);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.isNumber = isNumber;
function min(min, next) {
    return function (arg) {
        if (arg < min)
            return validation_result_1.error('LESS_THAN_MIN', arg + " is less than " + min);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.min = min;
function max(max, next) {
    return function (arg) {
        if (arg > max)
            return validation_result_1.error('GREATER_THAN_MAX', arg + " is greater than " + max);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.max = max;
function isString(next) {
    return function (arg) {
        if (typeof arg !== 'string')
            return validation_result_1.error('NOT_STRING', "Expected string, got " + typeof arg);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.isString = isString;
function matches(regex, next) {
    return function (arg) {
        if (!regex.test(arg))
            return validation_result_1.error('FAILED_REGEXP', "Failed regular expression " + regex.toString());
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.matches = matches;
function minLength(min, next) {
    return function (arg) {
        if (arg.length < min)
            return validation_result_1.error('LESS_THAN_MIN_LENGTH', "Length " + arg.length + " is less than " + min);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.minLength = minLength;
function maxLength(max, next) {
    return function (arg) {
        if (arg.length > max)
            return validation_result_1.error('GREATER_THAN_MAX_LENGTH', "Length " + arg.length + " is greater than " + max);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.maxLength = maxLength;
function lengthIs(length, next) {
    return function (arg) {
        if (arg.length !== length)
            return validation_result_1.error('LENGTH_NOT_EQUAL', "Length " + arg.length + " is not equal to " + length);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.lengthIs = lengthIs;
function isArray(next) {
    return function (arg) {
        if (!(arg instanceof Array))
            return validation_result_1.error('NOT_ARRAY', "Expected array, got " + typeof arg);
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.isArray = isArray;
function eachItem(assertion, next) {
    return function (arg) {
        var results = arg.map(function (item, index) { return utils_1.tryCatch(function () { return assertion(item); }, function (err) { return validation_result_1.errorFromException(err); }); });
        if (results.some(validation_result_1.ErrorResult.isErrorResult)) {
            return new validation_result_1.ErrorResult(results
                .map(function (item, index) {
                if (!item.success)
                    item.addPathNode(new validation_result_1.ArrayIndexPathNode(index));
                return item;
            })
                .filter(validation_result_1.ErrorResult.isErrorResult)
                .reduce(function (errors, result) { return errors.concat(result.errors); }, []));
        }
        var mapped = results.map(function (result) { return result.value; });
        return next ? next(mapped) : validation_result_1.success(mapped);
    };
}
exports.eachItem = eachItem;
function isObject(next) {
    return function (arg) {
        if (typeof arg !== 'object' || arg instanceof Array)
            return validation_result_1.error('NOT_OBJECT', "Expected object, got " + (arg instanceof Array ? 'array' : typeof arg));
        return next ? next(arg) : validation_result_1.success(arg);
    };
}
exports.isObject = isObject;
function equals(value) {
    var values = [];
    for (var _i = 1; _i < arguments.length; _i++) {
        values[_i - 1] = arguments[_i];
    }
    var vals = [value].concat(values);
    return function (arg) {
        for (var _i = 0, vals_1 = vals; _i < vals_1.length; _i++) {
            var val = vals_1[_i];
            if (val === arg)
                return validation_result_1.success(arg);
        }
        return validation_result_1.error('NOT_EQUAL', vals.length === 1 ? "'" + arg + "' does not equal '" + vals[0] + "'" : "'" + arg + "' not one of: " + vals.join(', '));
    };
}
exports.equals = equals;
function isMap(next) {
    return isObject(function (arg) {
        var nonStringKeys = utils_1.keysOf(arg).filter(function (key) { return typeof key !== 'string'; });
        if (nonStringKeys.length > 0) {
            return validation_result_1.error('NOT_STRING_KEY', "Expected string keys, got: " + nonStringKeys.map(function (key) { return key + " (" + typeof key + ")"; }));
        }
        return next ? next(arg) : validation_result_1.success(arg);
    });
}
exports.isMap = isMap;
function eachValue(assertion, next) {
    return function (arg) {
        var result = conformsTo(Object.keys(arg).reduce(function (validator, key) {
            validator[key] = assertion;
            return validator;
        }, {}))(arg);
        if (result.success && next)
            return next(result.value);
        return result;
    };
}
exports.eachValue = eachValue;
function either() {
    var assertions = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        assertions[_i] = arguments[_i];
    }
    return function (arg) {
        var errors = [];
        for (var _i = 0, assertions_1 = assertions; _i < assertions_1.length; _i++) {
            var assertion = assertions_1[_i];
            var result = assertion(arg);
            if (result.success) {
                return result;
            }
            errors = errors.concat(result.errors);
        }
        return validation_result_1.error('NO_MATCH', 'No match found - the following assertions failed:\n' + errors.map(function (error) { return utils_1.increaseIndent(error.toString(), 2); }).join('\n'));
    };
}
exports.either = either;
//# sourceMappingURL=index.js.map